'****************************************************************
'*  Name    : POWER CURVE LINEARISATION & AUTOMATIC SWR         *
'*  Author  : Dominique F1FRV                                   *
'*  Revision: 1                                                 *
'*  Date    : Octobre 2019                                      *
'****************************************************************
'
' Linarisation d'chelle de puissance & calcul automatique du ROS
' plac derrire un pont de mesure de ROS (SWR bridge) 
' Programme: " LINEARISATION_REV1.BAS "
' Rev hardware & software 10/2019.
' PIC 16LF1827 or 16F1847 en oscillateur interne  32 MHz
'
'/////////////////////////////////////////////////////////////////
'
' Entres:
'
' 1/ Pin 1 RA2: Mesure de la tension reprsentant la puissance directe
'               chelle quadratique, mais l'on s'en moque, 0-5 V.
'
' 2/ Pin 2 RA3: Mesure de la tension reprsentant la puissance rflchie
'               chelle quadratique, mais l'on s'en moque, 0-5 V.
'
' Sorties:
'
' 1/ Pin 16 RA7: Sortie PWM2 avec linarisation d'chelle pour puissance (prcision de linarisation ~2%)
'                Affiche la puissance sur une echelle 0  5 V  
'                Cette valeur de sortie sera ajuste en externe par un ampli oprationnel
'                pour voltmtre 1, 2, 3, 5, 10 ou 20 V pour 1, 2, 3, 5, 10 ou 20 kW.
'
' 2/ Pin 9 RB3 : Sortie PWM1 avec linarisation d'chelle pour ROS
'                Affiche le ROS sur une echelle 0  5 V ( ROS 1/1 = 0.5 V, ROS 10/1 = 5 V ). 
'                Cette valeur de sortie sera multiplie par 2 en externe par un ampli oprationnel,
'                pour voltmtre 0-10 V  SWR 1/1  10/1  SWR 1/1 = 1 V, SWR 10/1 = 10 V
'
'/////////////////////////////////////////////////////////////////
'
' Description SIMPLIFIEE de la routine de calcul:
'
' Mesurer la tension de puissance directe, V_PWR 0  5 V
'   La prcision sur 10 bit est de 5V/1024 (~5 mV), ~1 Watt sur echelle 5V = 1 kW.
'   Appliquer la formule de linarisation:  V_PWR_Lin = ((V_PWR x V_PWR)+0,25) / 5
'   Envoyer le rsultat sur le PWM pour sortie 0  5V.
'   Sortie 0  5 V sera adapte par ampli operationnel  l'chelle du voltmetre.
'
' Mesurer la tension de puissance rflchie, V_SWR 0  5 V
'   La prcision sur 10 bit est de 5V/1024 (~5 mV)
'   Appliquer la formule de calcul:  SWR_Lin = (V_PWR + V_SWR) / (V_PWR - V_SWR)
'   Envoyer le rsultat sur le PWM pour sortie 0  5V pour ROS maxi de 10/1.
'   Sortie 0  5 V sera double par ampli operationnel pour voltmetre 0  10V.
'
'/////////////////////////////////////////////////////////////////

		Device = 16LF1827
        Xtal = 32
    
      ; bits       76543210
        OSCCON  = %11110010             ; (Page 65 de la DS) Set HFIntosc at 32 Mhz
        T1CON   = %01101101             ; (Page 185 de la DS)
        T2CON   = %00000100             ; (Page 191 de la DS) 
        TRISA   = %00111111             ; (Page 122) Ports A6 et A7 en sortie (1 = Input, 0 = Output)
        ANSELA  = %00001111             ; (Page 123) Ports A0 to A3 en entre (1 = Input, 0 = Output)
        TRISB   = %01010111             ; (Page 127) Ports B3 et B7 en sortie (1 = Input, 0 = Output)
        ADCON0  = %00000011             ; (Page 145 de la DS) AN0
        ADCON1  = %11110000			    ; (Page 146 de la DS) 
        CCP1CON = %00111100             ; (Page 226 de la DS)
        CCP2CON = %00111100             ; (Page 226 de la DS)
        CCPTMRS = %00000000             ; (Page 227 de la DS)
        PWM1CON = %10000001             ; (Page 229 de la DS)  delay count 1 
        PWM2CON = %10000001             ; (Page 229 de la DS)  delay count 1
        PIE1    = %11111111             ; (Page 87 de la DS)
        PIE2    = %11111001             ; (Page 88 de la DS)
        PIE3    = %00000000             ; (Page 89 de la DS)
       
       
        Declare Serial_Baud 19200 
        Declare Rsin_Pin PORTB.6 
        Declare Rsin_Mode 1             ; Invers
        Declare Rsout_Pin PORTB.7
        Declare Rsout_Mode 0            ; Non invers, car l'inversion est ralise par Q1
        Declare CCP1_Pin PORTB.3        ; Enhanced PWM1 output Max 31.2 kHz @ 10 Bit, 125 kHz @ 8 bit !
        Declare CCP2_Pin PORTA.7        ; Enhanced PWM2 output Max 31.2 kHz @ 10 Bit, 125 kHz @ 8 Bit !
        
        Declare PortB_Pullups = 1       ; Pull up sur ports B
        
        Declare Adin_Res = 10 		    ; ADC 10 bit result required (0  1023)
		Declare Adin_Tad = FRC 		    ; Internal HF OSC chosen for ADC timer
		Declare Adin_Stime = 50         ; Allow 50 us sample time
        
		;Dim PORTC As PORTB              ; Remove ; If necessary for compiler
        Dim i As  Byte    		        ; Pour sub routine Read_PWR (0  255)
        Dim i_1 As Byte                 ; Pour sub routine Read_SWR (0  255)
        Dim V_PWR_1 As Word             ; i x 10 Bit ADC result (0  1023) stored in 16 Bit (0 To 65535)
        Dim V_PWR_2 As Word             ; Valeur moyenne sur i mesures (0 To 65535)
        Dim V_PWR As Word               ; Valeur moyenne sur i mesures EN VOLTS
        Dim Calcul_PWR_a As Word        ; Calcul intermdiaire (3 oprations maxi par ligne)
        Dim Calcul_PWR As Word          ; Sera utilis par le PWM pour sortie 0  5 V
        Dim PWR_Lin As Word             ; 5 V = 100% = 65535 vers le PWM pour sortie 0  5 V
        ;Dim PWR_Lin As Byte             ; 5 V = 100% = 255 vers le PWM pour sortie 0  5 V
        Dim PWR_Lin100 As Word          ; Pour sortie puissance relative 0  100% (100% = 65535)
        Dim V_SWR_1 As Word             ; i x 10 bit ADC result (0  1023) stored in 16 bit (0 to 65535)
        Dim V_SWR_2 As Word             ; Valeur moyenne sur i mesures (0 To 65535)
        Dim V_SWR As Word               ; Valeur moyenne sur i mesures EN VOLTS
        Dim Calcul_ROS_a As Word        ; Calcul intermdiaire du ROS (3 oprations maxi par ligne)
        Dim Calcul_ROS_b As Word        ; Calcul intermdiaire du ROS (3 oprations maxi par ligne)
        Dim Calcul_ROS As Word          ; Calcul du ROS: ROS 1/1=0.5V , ROS 10/1=5 V
        Dim SWR_Lin As Word             ; 5 V = 100% = 65535 vers le PWM pour sortie 0  5 V
        ;Dim SWR_Lin As Byte             ; 5 V = 100% = 255 vers le PWM pour sortie 0  5 V       
      
;;;;;;;;;;;;;;;:::::::;;;; PROGRAM ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
        
        DelayMS 100
        RSOut "  " ,13,10
        RSOut " POWER CURVE LINEARISATION & AUTOMATIC SWR " ,13,10
        RSOut " With PIC 16LF1827 or 16(L)F1847 " ,13,10
        RSOut " by Dominique, F1FRV " ,13,10
        RSOut " October 2019  Rev 1 " ,13,10
        RSOut "  " ,13,10
        DelayMS 100

Loop1:  
        GoSub Read_PWR                   ; On va lire et afficher la puissance
        GoSub Read_SWR                   ; On va lire et afficher le ROS                                      
        GoTo Loop1
        
;;;;;;;;;;;;;;;:::::::;;;; Sub Routines ;;;;;;;; Use by GoSub ;;;;;;;;;;;;;;;;;;;;;;

Read_PWR:                                ; sortie 0 a 5 V de la puissance
        DelayMS 5                        ; On attend x ms
        Clear V_PWR_1                    ; On efface la mmoire
        i=0
        For i = 0 To 9                   ; boucle pour 10 mesures
        V_PWR_1 = ADIn 0		         ; Store 10 x ADC 10 bit value (10230) to V_PWR_1 in 16 bit (65535)
        Next     
        V_PWR_2 = V_PWR_1 / 10           ; On divise pour valeur moyenne sur 10 mesures
        V_PWR = V_PWR_2 / 13107          ; Valeur en VOLTS (65535 / 13107 = 5)
        
        RSOut " V_PWR = " , Dec3 V_PWR, 13,10
        DelayMS 5    
        
        Calcul_PWR_a = ( V_PWR * V_PWR )               ; Mise au carr de V_PWR
        Calcul_PWR = (Calcul_PWR_a + 0.25 ) / 5        ; Pour linarisation de l'chelle quadratique (+/- 2%)                                            
        If Calcul_PWR >= 5 Then Calcul_PWR = 5         ; Sinon au dessus de 5 V ca passe a 0
        
        RSOut " Calcul_PWR kW = " , Dec3 Calcul_PWR, 13,10  ; Bas sur 5 V = 5 kW
        DelayMS 5
        
        PWR_Lin100 = ( Calcul_PWR / 5 ) * 100          ; Affichage puissance RELATIVE en % de 0  100
        
        RSOut " RELATIVE PWR % = " , Dec PWR_Lin100, 13,10
        DelayMS 5
                                                       ; 8 Bit PWM:
        PWR_Lin = Calcul_PWR * 51                      ; 51 = 255/5 pour avoir 5 V = 255
        If PWR_Lin >= 255 Then PWR_Lin = 255           ; Sinon au dessus de 255 ca passe a 0
                                                        ; 10 Bit PWM
        ;PWR_Lin = Calcul_PWR * 204.6                   ; 204.6 = 1023/5 pour avoir 5 V = 1023
        ;If PWR_Lin >= 1023 Then PWR_Lin = 1023         ; Sinon au dessus de 1023 ca passe a 0
        
        RSOut " PWR_Lin 8 bit = " , Dec PWR_Lin, 13,10
        ;RSOut " PWR_Lin 10 bit = " , Dec PWR_Lin, 13,10
        RSOut "  " ,13,10
        
        HPWM 2, PWR_Lin, 19200                         ; Sortie LINEARISEE 0  5 kW de 0  5 V
        Return 
          

Read_SWR:                                ; Sortie 0 a 5 V du ROS (Maxi 10/1)
        DelayMS 5                        ; On attend x ms
        Clear V_SWR_1                    ; On efface la mmoire
        i_1=0
        For i_1 = 0 To 9                 ; Boucle pour 10 mesures
        V_SWR_1 = ADIn 1		         ; Store 10 x ADC 10 bit value to V_SWR_1 in 16 bit
        Next     
        V_SWR_2 = V_SWR_1 / 10           ; On divise pour valeur moyenne sur 10 mesures
        V_SWR = V_SWR_2 / 13107          ; Valeur en VOLTS (65535 / 13107 = 5)
        ;V_SWR = V_SWR_2 / 51             ; Valeur en VOLTS (255 / 51 = 5)
        
        RSOut " V_SWR = " , Dec3 V_SWR, 13,10          ; Affichage V_SWR avec 3 dcimales
        DelayMS 5
                                                             
        Calcul_ROS_a = V_PWR + V_SWR                   ; Calcul intermdiaire (3 oprations maxi par ligne)
        Calcul_ROS_b = V_PWR - V_SWR                   ; Calcul intermdiaire (3 oprations maxi par ligne)
        Calcul_ROS = (Calcul_ROS_a / Calcul_ROS_b)     ; Calcul ROS
        If Calcul_ROS >= 10 Then Calcul_ROS = 10       ; Pour ROS 10 en fin d'chelle
        If Calcul_ROS <= 1 Then Calcul_ROS = 1         ; Sinon en dessous de 1 c'est impossible
        RSOut " ROS = " , Dec2 Calcul_ROS, 13,10       ; Affichage ROS avec 2 dcimales
        DelayMS 5
                                                       ; 8 Bit PWM:
        SWR_Lin = Calcul_ROS * 25.5                    ; 25.5 = 255/10 pour avoir ROS 10 = 5 V = 255
        If SWR_Lin >= 255 Then SWR_Lin = 255           ; Sinon au dessus de 255 ca passe a 0
        If SWR_Lin <= 26 Then SWR_Lin = 26             ; Sinon en dessous de 26 c'est impossible
                                                        ; 10 Bit PWM
        ;SWR_Lin = Calcul_ROS * 102.3                   ; 102.3 = 1023/10 pour avoir ROS 10 = 5 V = 1023
        ;If SWR_Lin >= 1023 Then SWR_Lin = 1023         ; Sinon au dessus de 1023 ca passe a 0
        ;If SWR_Lin <= 103 Then SWR_Lin = 103           ; Sinon en dessous de 103 c'est impossible
        
        RSOut " SWR 8 bit = " , Dec SWR_Lin, 13,10
        ;RSOut " SWR 10 bit = " , Dec SWR_Lin, 13,10  
        RSOut "  " ,13,10
        
        HPWM 1, SWR_Lin, 19200                          ; Sortie linaire du ROS de 1  10 (0.5  5 V)
        Return 
        
                             
